package com.zhentao.handler;

import org.apache.ibatis.type.JdbcType;
import org.apache.ibatis.type.TypeHandler;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.sql.*;

public class PointTypeHandler implements TypeHandler<String> {
    private static final Logger log = LoggerFactory.getLogger(PointTypeHandler.class);
    // 插入时将WKT字符串转换为POINT
    @Override
    public void setParameter(PreparedStatement ps, int i, String wkt, JdbcType jdbcType) throws SQLException {
        if (wkt == null) {
            ps.setNull(i, Types.OTHER);
        } else {
            // 关键修改：直接设置WKT字符串，不使用函数转换
            ps.setString(i, wkt);
            // 指定MySQL的GEOMETRY类型（4001）
            ps.setObject(i, wkt, 4001);
        }
    }

    // 查询时将POINT转换为WKT字符串
    @Override
    public String getResult(ResultSet rs, String columnName) throws SQLException {
        return rs.getString(columnName);
    }

    @Override
    public String getResult(ResultSet rs, int columnIndex) throws SQLException {
        return rs.getString(columnIndex);
    }

    @Override
    public String getResult(CallableStatement cs, int columnIndex) throws SQLException {
        return cs.getString(columnIndex);
    }
}