package com.zhentao.user.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.zhentao.tool.TokenUtils;
import com.zhentao.user.domain.UserOnlineStatus;
import com.zhentao.user.mapper.UserOnlineStatusMapper;
import com.zhentao.user.service.UserOnlineStatusService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import java.util.Date;
import java.util.concurrent.TimeUnit;

/**
* @author 31810
* @description 针对表【user_online_status(用户在线状态表)】的数据库操作Service实现
* @createDate 2025-06-18 15:27:28
*/
@Service
public class UserOnlineStatusServiceImpl extends ServiceImpl<UserOnlineStatusMapper, UserOnlineStatus>
    implements UserOnlineStatusService{
    private static final String KEY_ONLINE_USERS = "online_users";
    private static final String KEY_USER_STATUS = "user_status:%s";
    @Autowired
    private UserOnlineStatusMapper userOnlineStatusMapper;
    @Autowired
    private RedisTemplate<String, Object> redisTemplate;
    // 用户上线
    @Override
    public void userGoOnline(Long id) {
        redisTemplate.opsForSet().add(KEY_ONLINE_USERS, id.toString());
        String statusKey=KEY_USER_STATUS+id;
        redisTemplate.opsForHash().put(statusKey,"online_time",System.currentTimeMillis());
        redisTemplate.opsForHash().put(statusKey,"last_heartbeat",System.currentTimeMillis());
        redisTemplate.expire(statusKey,30, TimeUnit.HOURS);
        QueryWrapper<UserOnlineStatus> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", id);
        UserOnlineStatus onlineStatus = userOnlineStatusMapper.selectOne(queryWrapper);
        if(onlineStatus == null){
            UserOnlineStatus userOnlineStatus = new UserOnlineStatus();
            userOnlineStatus.setUserId(id);
            userOnlineStatus.setIsOnline(1);
            userOnlineStatus.setOnlineTime(new Date());
            userOnlineStatus.setLastHeartbeatTime(new Date());
            userOnlineStatusMapper.insert(userOnlineStatus);
            return;
        }
        onlineStatus.setIsOnline(1);
        onlineStatus.setOnlineTime(new Date());
        userOnlineStatusMapper.updateById(onlineStatus);
    }
    // 处理心跳包
    @Override
    public void handleHeartbeat(HttpServletRequest httpRequest) {
        String token = httpRequest.getHeader("token");
        Long uid= Long.valueOf(TokenUtils.getUserIdFromToken(token));
        String statusKey =KEY_USER_STATUS + uid;
        long l = System.currentTimeMillis();
        redisTemplate.opsForHash().put(statusKey, "last_heartbeat", l);
        redisTemplate.opsForSet().add(KEY_ONLINE_USERS, uid.toString());
    }

    @Override
    public void userGoOffline(Long id) {
        redisTemplate.opsForSet().remove(KEY_ONLINE_USERS, id.toString());
        QueryWrapper<UserOnlineStatus> queryWrapper = new QueryWrapper<>();
        queryWrapper.eq("user_id", id);
        UserOnlineStatus onlineStatus = userOnlineStatusMapper.selectOne(queryWrapper);
        onlineStatus.setIsOnline(0);
        onlineStatus.setOnlineTime(new Date());
        userOnlineStatusMapper.updateById(onlineStatus);
    }

}




